@@ -17,16 +17,14 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 #
-import asyncio
 from unittest import mock
-from unittest.mock import AsyncMock, MagicMock, Mock
+from unittest.mock import AsyncMock, Mock
 
 import pytest
 from nats.aio.client import Msg
 from nucliadb_protos.nodewriter_pb2 import IndexMessage, TypeMessage
 
 from nucliadb_node.listeners import IndexedPublisher
-from nucliadb_node.listeners.gc_scheduler import ShardManager
 from nucliadb_node.pull import Worker
 from nucliadb_utils import const
 
@@ -40,58 +38,6 @@ def pubsub():
         yield pubsub
 
 
-class TestShardManager:
-    @pytest.fixture()
-    def gc_lock(self):
-        yield asyncio.Semaphore(1)
-
-    @pytest.fixture()
-    def writer(self):
-        writer = MagicMock(garbage_collector=AsyncMock())
-        yield writer
-
-    @pytest.fixture()
-    def shard_manager(self, writer, gc_lock):
-        sm = ShardManager("shard_id", writer, gc_lock)
-        sm.target_gc_resources = 5
-        yield sm
-
-    @pytest.mark.asyncio
-    async def test_gc(self, shard_manager: ShardManager, writer):
-        await shard_manager.gc()
-
-        writer.garbage_collector.assert_awaited_once()
-
-    @pytest.mark.asyncio
-    async def test_schedule_gc(self, shard_manager: ShardManager, writer):
-        shard_manager.shard_changed_event(0)
-
-        await asyncio.sleep(0.1)
-
-        writer.garbage_collector.assert_awaited_once()
-
-    @pytest.mark.asyncio
-    async def test_schedule_gc_after_target(self, shard_manager: ShardManager, writer):
-        for _ in range(shard_manager.target_gc_resources):
-            shard_manager.shard_changed_event()
-
-        await asyncio.sleep(0.1)
-
-        writer.garbage_collector.assert_awaited_once()
-
-    @pytest.mark.asyncio
-    async def test_schedule_gc_after_target_multiple_times(
-        self, shard_manager: ShardManager, writer
-    ):
-        for _ in range(shard_manager.target_gc_resources):
-            shard_manager.shard_changed_event()
-            shard_manager.shard_changed_event()
-
-        await asyncio.sleep(0.1)
-
-        writer.garbage_collector.assert_awaited_once()
-
-
 class TestIndexedPublisher:
     @pytest.fixture(scope="function")
     async def publisher(self):
