@@ -24,7 +24,7 @@ use std::sync::Arc;
 use futures::Future;
 use nucliadb_core::metrics::replication as replication_metrics;
 use nucliadb_core::tracing::{debug, error, info, warn};
-use nucliadb_core::{metrics, Error, NodeResult};
+use nucliadb_core::{metrics, Channel, Error, NodeResult};
 use nucliadb_protos::prelude::EmptyQuery;
 use nucliadb_protos::replication;
 use tokio::io::AsyncWriteExt;
@@ -32,9 +32,9 @@ use tokio::sync::Semaphore;
 use tokio::time::Duration; // Import the Future trait
 use tonic::Request;
 
+use crate::cache::ShardWriterCache;
 use crate::replication::health::ReplicationHealthManager;
 use crate::settings::Settings;
-use crate::shards::cache::ShardWriterCache;
 use crate::shards::metadata::ShardMetadata;
 use crate::shards::writer::ShardWriter;
 use crate::utils::{list_shards, set_primary_node_id};
@@ -264,9 +264,10 @@ pub async fn connect_to_primary_and_replicate(
                 let metadata = ShardMetadata::new(
                     shards_path.join(shard_id.clone()),
                     shard_state.shard_id.clone(),
-                    Some(shard_state.kbid.clone()),
+                    shard_state.kbid.clone(),
                     shard_state.similarity.clone().into(),
-                    None,
+                    Channel::from(shard_state.release_channel),
+                    shard_state.normalize_vectors,
                 );
                 let shard_cache_clone = Arc::clone(&shard_cache);
 
