@@ -30,9 +30,9 @@ use tokio::io::AsyncReadExt;
 use tokio_stream::wrappers::ReceiverStream;
 use tonic::Response;
 
+use crate::cache::ShardWriterCache;
 use crate::replication::NodeRole;
 use crate::settings::Settings;
-use crate::shards::cache::ShardWriterCache;
 use crate::shards::metadata::Similarity;
 use crate::shards::writer::ShardWriter;
 use crate::utils::{get_primary_node_id, list_shards, read_host_key};
@@ -229,6 +229,7 @@ impl replication::replication_service_server::ReplicationService for Replication
                         generation_id: gen_id,
                         kbid: metadata.kbid().unwrap_or_default(),
                         similarity: similarity.to_string(),
+                        normalize_vectors: metadata.normalize_vectors(),
                     });
                 }
             } else {
