@@ -16,9 +16,10 @@
 // You should have received a copy of the GNU Affero General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+use super::indexes::ShardIndexes;
+use super::metadata::ShardMetadata;
+use super::versioning::Versions;
 use crate::disk_structure::*;
-use crate::shards::metadata::ShardMetadata;
-use crate::shards::versions::Versions;
 use crate::telemetry::run_with_telemetry;
 use crossbeam_utils::thread as crossbeam_thread;
 use nucliadb_core::paragraphs::*;
@@ -26,12 +27,16 @@ use nucliadb_core::prelude::*;
 use nucliadb_core::protos;
 use nucliadb_core::protos::shard_created::{DocumentService, ParagraphService, RelationService, VectorService};
 use nucliadb_core::protos::{
-    DocumentSearchRequest, DocumentSearchResponse, EdgeList, GetShardRequest, ParagraphSearchRequest,
-    ParagraphSearchResponse, RelationPrefixSearchRequest, RelationSearchRequest, RelationSearchResponse, SearchRequest,
-    SearchResponse, Shard, ShardFile, ShardFileChunk, ShardFileList, StreamRequest, SuggestFeatures, SuggestRequest,
-    SuggestResponse, VectorSearchRequest, VectorSearchResponse,
+    DocumentSearchRequest, DocumentSearchResponse, EdgeList, ParagraphSearchRequest, ParagraphSearchResponse,
+    RelationPrefixSearchRequest, RelationSearchRequest, RelationSearchResponse, SearchRequest, SearchResponse, Shard,
+    ShardFile, ShardFileChunk, ShardFileList, StreamRequest, SuggestFeatures, SuggestRequest, SuggestResponse,
+    VectorSearchRequest, VectorSearchResponse,
 };
+use nucliadb_core::query_language::BooleanExpression;
+use nucliadb_core::query_language::BooleanOperation;
+use nucliadb_core::query_language::Operator;
 use nucliadb_core::query_planner;
+use nucliadb_core::query_planner::PreFilterRequest;
 use nucliadb_core::relations::*;
 use nucliadb_core::texts::*;
 use nucliadb_core::thread::*;
@@ -45,12 +50,47 @@ use std::collections::HashSet;
 use std::fs::{self, File};
 use std::io::{BufReader, Read};
 use std::path::{Path, PathBuf};
-use std::sync::Arc;
+use std::sync::{Arc, RwLock};
 
 const MAX_SUGGEST_COMPOUND_WORDS: usize = 3;
 const MIN_VIABLE_PREFIX_SUGGEST: usize = 1;
 const CHUNK_SIZE: usize = 65535;
 
+fn open_vectors_reader(version: u32, path: &Path) -> NodeResult<VectorsReaderPointer> {
+    match version {
+        1 => nucliadb_vectors::service::VectorReaderService::open(path)
+            .map(|i| Arc::new(RwLock::new(i)) as VectorsReaderPointer),
+        2 => nucliadb_vectors::service::VectorReaderService::open(path)
+            .map(|i| Arc::new(RwLock::new(i)) as VectorsReaderPointer),
+        v => Err(node_error!("Invalid vectors version {v}")),
+    }
+}
+fn open_paragraphs_reader(version: u32, path: &Path) -> NodeResult<ParagraphsReaderPointer> {
+    match version {
+        2 => nucliadb_paragraphs2::reader::ParagraphReaderService::open(path)
+            .map(|i| Arc::new(RwLock::new(i)) as ParagraphsReaderPointer),
+        3 => nucliadb_paragraphs3::reader::ParagraphReaderService::open(path)
+            .map(|i| Arc::new(RwLock::new(i)) as ParagraphsReaderPointer),
+        v => Err(node_error!("Invalid paragraphs version {v}")),
+    }
+}
+
+fn open_texts_reader(version: u32, path: &Path) -> NodeResult<TextsReaderPointer> {
+    match version {
+        2 => nucliadb_texts2::reader::TextReaderService::open(path)
+            .map(|i| Arc::new(RwLock::new(i)) as TextsReaderPointer),
+        v => Err(node_error!("Invalid text reader version {v}")),
+    }
+}
+
+fn open_relations_reader(version: u32, path: &Path) -> NodeResult<RelationsReaderPointer> {
+    match version {
+        2 => nucliadb_relations2::reader::RelationsReaderService::open(path)
+            .map(|i| Arc::new(RwLock::new(i)) as RelationsReaderPointer),
+        v => Err(node_error!("Invalid relations version {v}")),
+    }
+}
+
 pub struct ShardFileChunkIterator {
     file_path: PathBuf,
     reader: BufReader<File>,
@@ -106,16 +146,13 @@ pub struct ShardReader {
     paragraph_reader: ParagraphsReaderPointer,
     vector_reader: VectorsReaderPointer,
     relation_reader: RelationsReaderPointer,
-    document_version: i32,
-    paragraph_version: i32,
-    vector_version: i32,
-    relation_version: i32,
+    versions: Versions,
 }
 
 impl ShardReader {
     #[tracing::instrument(skip_all)]
     pub fn text_version(&self) -> DocumentService {
-        match self.document_version {
+        match self.versions.texts {
             0 => DocumentService::DocumentV0,
             1 => DocumentService::DocumentV1,
             2 => DocumentService::DocumentV2,
@@ -125,17 +162,18 @@ impl ShardReader {
 
     #[tracing::instrument(skip_all)]
     pub fn paragraph_version(&self) -> ParagraphService {
-        match self.paragraph_version {
+        match self.versions.paragraphs {
             0 => ParagraphService::ParagraphV0,
             1 => ParagraphService::ParagraphV1,
             2 => ParagraphService::ParagraphV2,
+            3 => ParagraphService::ParagraphV3,
             i => panic!("Unknown paragraph version {i}"),
         }
     }
 
     #[tracing::instrument(skip_all)]
     pub fn vector_version(&self) -> VectorService {
-        match self.vector_version {
+        match self.versions.vectors {
             0 => VectorService::VectorV0,
             1 => VectorService::VectorV1,
             i => panic!("Unknown vector version {i}"),
@@ -144,7 +182,7 @@ impl ShardReader {
 
     #[tracing::instrument(skip_all)]
     pub fn relation_version(&self) -> RelationService {
-        match self.relation_version {
+        match self.versions.relations {
             0 => RelationService::RelationV0,
             1 => RelationService::RelationV1,
             2 => RelationService::RelationV2,
@@ -154,7 +192,7 @@ impl ShardReader {
 
     #[measure(actor = "shard", metric = "get_info")]
     #[tracing::instrument(skip_all)]
-    pub fn get_info(&self, request: &GetShardRequest) -> NodeResult<Shard> {
+    pub fn get_info(&self) -> NodeResult<Shard> {
         let span = tracing::Span::current();
 
         let paragraphs = self.paragraph_reader.clone();
@@ -166,7 +204,7 @@ impl ShardReader {
         let info = info_span!(parent: &span, "paragraph count");
         let paragraph_task = || run_with_telemetry(info, move || read_rw_lock(&paragraphs).count());
         let info = info_span!(parent: &span, "vector count");
-        let vector_task = || run_with_telemetry(info, move || read_rw_lock(&vectors).count(&request.vectorset));
+        let vector_task = || run_with_telemetry(info, move || read_rw_lock(&vectors).count());
 
         let mut text_result = Ok(0);
         let mut paragraph_result = Ok(0);
@@ -180,7 +218,7 @@ impl ShardReader {
 
         Ok(Shard {
             metadata: Some(protos::ShardMetadata {
-                kbid: self.metadata.kbid().unwrap_or_default(),
+                kbid: self.metadata.kbid(),
                 release_channel: self.metadata.channel() as i32,
             }),
             shard_id: self.id.clone(),
@@ -223,32 +261,13 @@ impl ShardReader {
         let span = tracing::Span::current();
 
         let metadata = ShardMetadata::open(shard_path.to_path_buf())?;
-        let tsc = TextConfig {
-            path: shard_path.join(TEXTS_DIR),
-        };
+        let indexes = ShardIndexes::load(shard_path).unwrap_or_else(|_| ShardIndexes::new(shard_path));
 
-        let psc: ParagraphConfig = ParagraphConfig {
-            path: shard_path.join(PARAGRAPHS_DIR),
-        };
-
-        let channel = metadata.channel();
-
-        let vsc = VectorConfig {
-            similarity: None,
-            path: shard_path.join(VECTORS_DIR),
-            vectorset: shard_path.join(VECTORSET_DIR),
-            channel,
-            shard_id: id.clone(),
-        };
-        let rsc = RelationConfig {
-            path: shard_path.join(RELATIONS_DIR),
-            channel,
-        };
         let versions = Versions::load(&shard_path.join(VERSION_FILE))?;
-        let text_task = || Some(versions.get_texts_reader(&tsc));
-        let paragraph_task = || Some(versions.get_paragraphs_reader(&psc));
-        let vector_task = || Some(versions.get_vectors_reader(&vsc));
-        let relation_task = || Some(versions.get_relations_reader(&rsc));
+        let text_task = || Some(open_texts_reader(versions.texts, &indexes.texts_path()));
+        let paragraph_task = || Some(open_paragraphs_reader(versions.paragraphs, &indexes.paragraphs_path()));
+        let vector_task = || Some(open_vectors_reader(versions.vectors, &indexes.vectors_path()));
+        let relation_task = || Some(open_relations_reader(versions.relations, &indexes.relations_path()));
 
         let info = info_span!(parent: &span, "text open");
         let text_task = || run_with_telemetry(info, text_task);
@@ -285,10 +304,7 @@ impl ShardReader {
             paragraph_reader: paragraphs.unwrap(),
             vector_reader: vectors.unwrap(),
             relation_reader: relations.unwrap(),
-            document_version: versions.version_texts() as i32,
-            paragraph_version: versions.version_paragraphs() as i32,
-            vector_version: versions.version_vectors() as i32,
-            relation_version: versions.version_relations() as i32,
+            versions,
         })
     }
 
@@ -315,16 +331,45 @@ impl ShardReader {
 
     #[measure(actor = "shard", metric = "suggest")]
     #[tracing::instrument(skip_all)]
-    pub fn suggest(&self, request: SuggestRequest) -> NodeResult<SuggestResponse> {
+    pub fn suggest(&self, mut request: SuggestRequest) -> NodeResult<SuggestResponse> {
         let span = tracing::Span::current();
 
-        let suggest_paragraphs = request.features.contains(&(SuggestFeatures::Paragraphs as i32));
+        let mut suggest_paragraphs = request.features.contains(&(SuggestFeatures::Paragraphs as i32));
         let suggest_entities = request.features.contains(&(SuggestFeatures::Entities as i32));
 
         let paragraphs_reader_service = self.paragraph_reader.clone();
         let relations_reader_service = self.relation_reader.clone();
         let prefixes = Self::split_suggest_query(request.body.clone(), MAX_SUGGEST_COMPOUND_WORDS);
 
+        // Prefilter to apply field label filters
+        if let Some(filter) = &mut request.filter {
+            // nucliadb_paragraphs2 has all the labels and doesn't need a prefilter
+            if !filter.field_labels.is_empty() && suggest_paragraphs && self.versions.paragraphs != 2 {
+                let labels = std::mem::take(&mut filter.field_labels);
+                let operands = labels.into_iter().map(BooleanExpression::Literal).collect();
+                let op = BooleanOperation {
+                    operator: Operator::And,
+                    operands,
+                };
+                let prefilter = PreFilterRequest {
+                    timestamp_filters: vec![],
+                    security: None,
+                    formula: Some(BooleanExpression::Operation(op)),
+                };
+
+                let prefiltered = read_rw_lock(&self.text_reader).prefilter(&prefilter)?;
+
+                // Apply prefilter to paragraphs query
+                match prefiltered.valid_fields {
+                    query_planner::ValidFieldCollector::All => {}
+                    query_planner::ValidFieldCollector::Some(keys) => {
+                        request.key_filters = keys.iter().map(|v| format!("{}{}", v.resource_id, v.field_id)).collect()
+                    }
+                    query_planner::ValidFieldCollector::None => suggest_paragraphs = false,
+                }
+            }
+        }
+
         let suggest_paragraphs_task = suggest_paragraphs.then(|| {
             let paragraph_task = move || read_rw_lock(&paragraphs_reader_service).suggest(&request);
             let info = info_span!(parent: &span, "paragraph suggest");
@@ -406,7 +451,7 @@ impl ShardReader {
     #[measure(actor = "shard", metric = "request/search")]
     #[tracing::instrument(skip_all)]
     pub fn search(&self, search_request: SearchRequest) -> NodeResult<SearchResponse> {
-        let query_plan = query_planner::build_query_plan(self.paragraph_version, search_request)?;
+        let query_plan = query_planner::build_query_plan(self.versions.paragraphs, search_request)?;
 
         let search_id = uuid::Uuid::new_v4().to_string();
         let span = tracing::Span::current();
@@ -589,8 +634,8 @@ impl ShardReader {
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn vector_count(&self, vector_set: &str) -> NodeResult<usize> {
-        read_rw_lock(&self.vector_reader).count(vector_set)
+    pub fn vector_count(&self) -> NodeResult<usize> {
+        read_rw_lock(&self.vector_reader).count()
     }
 
     #[tracing::instrument(skip_all)]
