@@ -28,6 +28,8 @@ pub struct KnowledgeBoxConfig {
     pub disable_vectors: bool,
     #[prost(int64, tag="7")]
     pub migration_version: i64,
+    /// DEPRECATED: duplicated field also stored in `writer.proto Shards`
+    #[deprecated]
     #[prost(enumeration="super::utils::ReleaseChannel", tag="8")]
     pub release_channel: i32,
 }
@@ -39,6 +41,7 @@ pub struct KnowledgeBoxNew {
     pub slug: ::prost::alloc::string::String,
     #[prost(message, optional, tag="2")]
     pub config: ::core::option::Option<KnowledgeBoxConfig>,
+    /// this fields are only set by backend when creating hosted KBs
     #[prost(string, tag="3")]
     pub forceuuid: ::prost::alloc::string::String,
     #[prost(enumeration="super::utils::VectorSimilarity", tag="4")]
@@ -47,10 +50,16 @@ pub struct KnowledgeBoxNew {
     pub vector_dimension: ::core::option::Option<i32>,
     #[prost(float, optional, tag="6")]
     pub default_min_score: ::core::option::Option<f32>,
-    #[prost(enumeration="super::utils::ReleaseChannel", tag="7")]
-    pub release_channel: i32,
+    #[prost(uint32, repeated, tag="9")]
+    pub matryoshka_dimensions: ::prost::alloc::vec::Vec<u32>,
+    /// this field are only used by NucliaDB Writer API when creating a KB. Used
+    /// in onprem scenarios
     #[prost(string, tag="8")]
     pub learning_config: ::prost::alloc::string::String,
+    /// release channel, although not used when backend creates hosted KBs, it's
+    /// recomputed and changed depending on the environment
+    #[prost(enumeration="super::utils::ReleaseChannel", tag="7")]
+    pub release_channel: i32,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
 pub struct NewKnowledgeBoxResponse {
@@ -232,6 +241,10 @@ pub struct SemanticModelMetadata {
     pub vector_dimension: ::core::option::Option<i32>,
     #[prost(float, optional, tag="3")]
     pub default_min_score: ::core::option::Option<f32>,
+    /// list of possible subdivisions of the matryoshka embeddings (if the model
+    /// supports it)
+    #[prost(uint32, repeated, tag="4")]
+    pub matryoshka_dimensions: ::prost::alloc::vec::Vec<u32>,
 }
 // Do not update this model without confirmation of internal Learning Config API
 
