@@ -4,12 +4,16 @@ pub struct OpStatus {
     pub status: i32,
     #[prost(string, tag="2")]
     pub detail: ::prost::alloc::string::String,
+    #[deprecated]
     #[prost(uint64, tag="3")]
     pub field_count: u64,
+    #[deprecated]
     #[prost(uint64, tag="5")]
     pub paragraph_count: u64,
+    #[deprecated]
     #[prost(uint64, tag="6")]
     pub sentence_count: u64,
+    #[deprecated]
     #[prost(string, tag="4")]
     pub shard_id: ::prost::alloc::string::String,
 }
@@ -69,6 +73,9 @@ pub struct NewShardRequest {
     pub kbid: ::prost::alloc::string::String,
     #[prost(enumeration="super::utils::ReleaseChannel", tag="3")]
     pub release_channel: i32,
+    /// indicates whether the shard should normalize vectors on indexing or not
+    #[prost(bool, tag="4")]
+    pub normalize_vectors: bool,
 }
 #[derive(Clone, PartialEq, ::prost::Message)]
 pub struct NewVectorSetRequest {
@@ -293,6 +300,25 @@ pub mod node_writer_client {
             );
             self.inner.unary(request.into_request(), path, codec).await
         }
+        pub async fn set_resource_from_storage(
+            &mut self,
+            request: impl tonic::IntoRequest<super::IndexMessage>,
+        ) -> Result<tonic::Response<super::OpStatus>, tonic::Status> {
+            self.inner
+                .ready()
+                .await
+                .map_err(|e| {
+                    tonic::Status::new(
+                        tonic::Code::Unknown,
+                        format!("Service was not ready: {}", e.into()),
+                    )
+                })?;
+            let codec = tonic::codec::ProstCodec::default();
+            let path = http::uri::PathAndQuery::from_static(
+                "/nodewriter.NodeWriter/SetResourceFromStorage",
+            );
+            self.inner.unary(request.into_request(), path, codec).await
+        }
         pub async fn remove_resource(
             &mut self,
             request: impl tonic::IntoRequest<super::super::noderesources::ResourceId>,
@@ -436,6 +462,10 @@ pub mod node_writer_server {
             &self,
             request: tonic::Request<super::super::noderesources::Resource>,
         ) -> Result<tonic::Response<super::OpStatus>, tonic::Status>;
+        async fn set_resource_from_storage(
+            &self,
+            request: tonic::Request<super::IndexMessage>,
+        ) -> Result<tonic::Response<super::OpStatus>, tonic::Status>;
         async fn remove_resource(
             &self,
             request: tonic::Request<super::super::noderesources::ResourceId>,
@@ -747,6 +777,44 @@ pub mod node_writer_server {
                     };
                     Box::pin(fut)
                 }
+                "/nodewriter.NodeWriter/SetResourceFromStorage" => {
+                    #[allow(non_camel_case_types)]
+                    struct SetResourceFromStorageSvc<T: NodeWriter>(pub Arc<T>);
+                    impl<T: NodeWriter> tonic::server::UnaryService<super::IndexMessage>
+                    for SetResourceFromStorageSvc<T> {
+                        type Response = super::OpStatus;
+                        type Future = BoxFuture<
+                            tonic::Response<Self::Response>,
+                            tonic::Status,
+                        >;
+                        fn call(
+                            &mut self,
+                            request: tonic::Request<super::IndexMessage>,
+                        ) -> Self::Future {
+                            let inner = self.0.clone();
+                            let fut = async move {
+                                (*inner).set_resource_from_storage(request).await
+                            };
+                            Box::pin(fut)
+                        }
+                    }
+                    let accept_compression_encodings = self.accept_compression_encodings;
+                    let send_compression_encodings = self.send_compression_encodings;
+                    let inner = self.inner.clone();
+                    let fut = async move {
+                        let inner = inner.0;
+                        let method = SetResourceFromStorageSvc(inner);
+                        let codec = tonic::codec::ProstCodec::default();
+                        let mut grpc = tonic::server::Grpc::new(codec)
+                            .apply_compression_config(
+                                accept_compression_encodings,
+                                send_compression_encodings,
+                            );
+                        let res = grpc.unary(method, req).await;
+                        Ok(res)
+                    };
+                    Box::pin(fut)
+                }
                 "/nodewriter.NodeWriter/RemoveResource" => {
                     #[allow(non_camel_case_types)]
                     struct RemoveResourceSvc<T: NodeWriter>(pub Arc<T>);
