@@ -20,7 +20,6 @@
 use crate::data_point_provider::reader::Reader;
 use crate::data_point_provider::*;
 use crate::formula::{AtomClause, BooleanOperator, Clause, CompoundClause, Formula};
-use crate::indexset::ReaderSet;
 use crate::service::query_io;
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::prost::Message;
@@ -53,7 +52,6 @@ impl<'a> SearchRequest for (usize, &'a VectorSearchRequest, Formula) {
 
 pub struct VectorReaderService {
     index: Reader,
-    indexset: ReaderSet,
 }
 impl Debug for VectorReaderService {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -64,22 +62,13 @@ impl Debug for VectorReaderService {
 impl VectorReader for VectorReaderService {
     fn update(&mut self) -> NodeResult<()> {
         self.index.update()?;
-        self.indexset.update()?;
         Ok(())
     }
 
     #[tracing::instrument(skip_all)]
-    fn count(&self, vectorset: &str) -> NodeResult<usize> {
-        if vectorset.is_empty() {
-            debug!("Id for the vectorset is empty");
-            Ok(self.index.size())
-        } else if let Some(index) = self.indexset.get(vectorset)? {
-            debug!("Counting nodes for {vectorset}");
-            Ok(index.size())
-        } else {
-            debug!("There was not a set called {vectorset}");
-            Ok(0)
-        }
+    fn count(&self) -> NodeResult<usize> {
+        debug!("Id for the vectorset is empty");
+        Ok(self.index.size())
     }
 
     #[measure(actor = "vectors", metric = "search")]
@@ -93,7 +82,6 @@ impl VectorReader for VectorReaderService {
         let offset = offset as usize;
         let total_to_get = total_to_get as usize;
 
-        let key_filters = request.key_filters.iter().cloned().map(AtomClause::key_prefix);
         let field_labels = request.field_labels.iter().cloned().map(AtomClause::label);
         let mut formula = Formula::new();
 
@@ -103,10 +91,10 @@ impl VectorReader for VectorReaderService {
             formula.extend(field_clause);
         }
 
-        if key_filters.len() > 0 {
-            let clause_labels = key_filters.map(Clause::Atom).collect();
-            let key_clause = CompoundClause::new(BooleanOperator::Or, clause_labels);
-            formula.extend(key_clause);
+        if !request.key_filters.is_empty() {
+            let (field_ids, resource_ids) = request.key_filters.iter().cloned().partition(|k| k.contains('/'));
+            let clause_labels = AtomClause::key_set(resource_ids, field_ids);
+            formula.extend(clause_labels);
         }
 
         if let Some(filter) = context.filtering_formula.as_ref() {
@@ -118,16 +106,8 @@ impl VectorReader for VectorReaderService {
         let v = time.elapsed().as_millis();
         debug!("{id:?} - Searching: starts at {v} ms");
 
-        let result = if request.vector_set.is_empty() {
-            debug!("{id:?} - No vectorset specified, searching in the main index");
-            self.index.search(&search_request)?
-        } else if let Some(index) = self.indexset.get(&request.vector_set)? {
-            debug!("{id:?} - vectorset specified and found, searching on {}", request.vector_set);
-            index.search(&search_request)?
-        } else {
-            debug!("{id:?} - A was vectorset specified, but not found. {} is not a vectorset", request.vector_set);
-            vec![]
-        };
+        let result = self.index.search(&search_request)?;
+
         let v = time.elapsed().as_millis();
         debug!("{id:?} - Searching: ends at {v} ms");
         debug!("{id:?} - Creating results: starts at {v} ms");
@@ -194,12 +174,8 @@ impl VectorReaderService {
         if !config.path.exists() {
             return Err(node_error!("Invalid path {:?}", config.path));
         }
-        if !config.vectorset.exists() {
-            return Err(node_error!("Invalid path {:?}", config.vectorset));
-        }
         Ok(VectorReaderService {
             index: Reader::open(&config.path)?,
-            indexset: ReaderSet::new(&config.vectorset)?,
         })
     }
 }
@@ -224,8 +200,9 @@ mod tests {
         let vsc = VectorConfig {
             similarity: Some(VectorSimilarity::Cosine),
             path: dir.path().join("vectors"),
-            vectorset: dir.path().join("vectorset"),
             channel: Channel::EXPERIMENTAL,
+            shard_id: "abc".into(),
+            normalize_vectors: false,
         };
         let raw_sentences = [
             ("DOC/KEY/1/1".to_string(), vec![1.0, 3.0, 4.0]),
@@ -306,8 +283,9 @@ mod tests {
         let vsc = VectorConfig {
             similarity: Some(VectorSimilarity::Cosine),
             path: dir.path().join("vectors"),
-            vectorset: dir.path().join("vectorset"),
             channel: Channel::EXPERIMENTAL,
+            shard_id: "abc".into(),
+            normalize_vectors: false,
         };
         let raw_sentences = [
             ("DOC/KEY/1/1".to_string(), vec![1.0, 3.0, 4.0]),
@@ -387,7 +365,7 @@ mod tests {
             ..Default::default()
         };
         let result = reader.search(&request, &context).unwrap();
-        let no_nodes = reader.count("").unwrap();
+        let no_nodes = reader.count().unwrap();
         assert_eq!(no_nodes, 4);
         assert_eq!(result.documents.len(), 3);
 
@@ -404,7 +382,7 @@ mod tests {
             ..Default::default()
         };
         let result = reader.search(&request, &context).unwrap();
-        let no_nodes = reader.count("").unwrap();
+        let no_nodes = reader.count().unwrap();
         assert_eq!(no_nodes, 4);
         assert_eq!(result.documents.len(), 0);
 
@@ -427,8 +405,9 @@ mod tests {
         let vsc = VectorConfig {
             similarity: Some(VectorSimilarity::Cosine),
             path: dir.path().join("vectors"),
-            vectorset: dir.path().join("vectorset"),
             channel: Channel::EXPERIMENTAL,
+            shard_id: "abc".into(),
+            normalize_vectors: false,
         };
         let raw_sentences =
             [("DOC/KEY/1/1".to_string(), vec![1.0, 2.0, 3.0]), ("DOC/KEY/1/2".to_string(), vec![1.0, 2.0, 3.0])];
