@@ -21,33 +21,26 @@ use crate::data_point::{self, DataPointPin, Elem, LabelDictionary};
 use crate::data_point_provider::garbage_collector;
 use crate::data_point_provider::writer::Writer;
 use crate::data_point_provider::*;
-use crate::indexset::IndexKeyCollector;
-use crate::indexset::WriterSet;
-use nucliadb_core::metrics::{request_time, vectors::MergeSource};
+use crate::utils;
+use nucliadb_core::metrics;
+use nucliadb_core::metrics::request_time;
+use nucliadb_core::metrics::vectors::MergeSource;
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::prost::Message;
 use nucliadb_core::protos::resource::ResourceStatus;
-use nucliadb_core::protos::{Resource, ResourceId, VectorSetId, VectorSimilarity};
+use nucliadb_core::protos::{Resource, ResourceId};
 use nucliadb_core::tracing::{self, *};
 use nucliadb_core::vectors::MergeMetrics;
 use nucliadb_core::vectors::*;
-use nucliadb_core::{metrics, IndexFiles, RawReplicaState};
+use nucliadb_core::{IndexFiles, RawReplicaState};
 use nucliadb_procs::measure;
 use std::collections::HashMap;
 use std::fmt::Debug;
-use std::fs;
 use std::time::Instant;
 use std::time::SystemTime;
 
-impl IndexKeyCollector for Vec<String> {
-    fn add_key(&mut self, key: String) {
-        self.push(key);
-    }
-}
-
 pub struct VectorWriterService {
     index: Writer,
-    indexset: WriterSet,
     config: VectorConfig,
 }
 
@@ -78,56 +71,6 @@ impl VectorWriter for VectorWriterService {
         Ok(merge_result.get_metrics())
     }
 
-    #[measure(actor = "vectors", metric = "list_vectorsets")]
-    #[tracing::instrument(skip_all)]
-    fn list_vectorsets(&self) -> NodeResult<Vec<String>> {
-        let time = Instant::now();
-
-        let id: Option<String> = None;
-        let mut collector = Vec::new();
-        self.indexset.index_keys(&mut collector);
-
-        let took = time.elapsed().as_secs_f64();
-        debug!("{id:?} - Ending at {took} ms");
-
-        Ok(collector)
-    }
-
-    #[measure(actor = "vectors", metric = "add_vectorset")]
-    #[tracing::instrument(skip_all)]
-    fn add_vectorset(&mut self, setid: &VectorSetId, similarity: VectorSimilarity) -> NodeResult<()> {
-        let time = Instant::now();
-
-        let id = setid.shard.as_ref().map(|s| &s.id);
-        let set = &setid.vectorset;
-        let indexid = setid.vectorset.as_str();
-        let similarity = similarity.into();
-        self.indexset.create_index(indexid, similarity)?;
-        self.indexset.commit()?;
-
-        let took = time.elapsed().as_secs_f64();
-        debug!("{id:?}/{set} - Ending at {took} ms");
-
-        Ok(())
-    }
-
-    #[measure(actor = "vectors", metric = "remove_vectorset")]
-    #[tracing::instrument(skip_all)]
-    fn remove_vectorset(&mut self, setid: &VectorSetId) -> NodeResult<()> {
-        let time = Instant::now();
-
-        let id = setid.shard.as_ref().map(|s| &s.id);
-        let set = &setid.vectorset;
-        let index_id = &setid.vectorset;
-        self.indexset.remove_index(index_id);
-        self.indexset.commit()?;
-
-        let took = time.elapsed().as_secs_f64();
-        debug!("{id:?}/{set} - Ending at {took} ms");
-
-        Ok(())
-    }
-
     #[measure(actor = "vectors", metric = "reload")]
     #[tracing::instrument(skip_all)]
     fn reload(&mut self) -> NodeResult<()> {
@@ -170,21 +113,26 @@ impl VectorWriter for VectorWriterService {
         let temporal_mark = SystemTime::now();
         let mut lengths: HashMap<usize, Vec<_>> = HashMap::new();
         let mut elems = Vec::new();
+        let normalize_vectors = self.index.metadata().normalize_vectors;
         if resource.status != ResourceStatus::Delete as i32 {
-            for (paragraph_field, paragraph) in resource.paragraphs.iter() {
-                for index in paragraph.paragraphs.values() {
-                    let mut inner_labels = index.labels.clone();
-                    inner_labels.push(paragraph_field.clone());
+            for (field_id, field_paragraphs) in resource.paragraphs.iter() {
+                for paragraph in field_paragraphs.paragraphs.values() {
+                    let mut inner_labels = paragraph.labels.clone();
+                    inner_labels.push(field_id.clone());
                     let labels = LabelDictionary::new(inner_labels);
 
-                    for (key, sentence) in index.sentences.iter().clone() {
+                    for (key, sentence) in paragraph.sentences.iter().clone() {
                         let key = key.to_string();
                         let labels = labels.clone();
-                        let vector = sentence.vector.clone();
+                        let vector = if normalize_vectors {
+                            utils::normalize_vector(&sentence.vector)
+                        } else {
+                            sentence.vector.clone()
+                        };
                         let metadata = sentence.metadata.as_ref().map(|m| m.encode_to_vec());
                         let bucket = lengths.entry(vector.len()).or_default();
                         elems.push(Elem::new(key, vector, labels, metadata));
-                        bucket.push(paragraph_field);
+                        bucket.push(field_id);
                     }
                 }
             }
@@ -218,56 +166,6 @@ impl VectorWriter for VectorWriterService {
         let v = time.elapsed().as_millis();
         debug!("{id:?} - Main index set resource: ends {v} ms");
 
-        let v = time.elapsed().as_millis();
-        debug!("{id:?} - Set resource in vector sets: starts {v} ms");
-
-        let mut writer_sets = HashMap::new();
-        for (index_key, vectors_to_delete) in resource.vectors_to_delete.iter() {
-            let Some(mut writer) = self.indexset.get(index_key)? else {
-                continue;
-            };
-
-            for vector_key in &vectors_to_delete.vectors {
-                writer.record_delete(vector_key.as_bytes(), temporal_mark);
-            }
-
-            writer_sets.insert(index_key.clone(), writer);
-        }
-
-        for (index_key, vectors_to_add) in resource.vectors.iter() {
-            if !writer_sets.contains_key(index_key) {
-                let Some(writer) = self.indexset.get(index_key)? else {
-                    continue;
-                };
-                writer_sets.insert(index_key.clone(), writer);
-            }
-
-            let writer = writer_sets.get_mut(index_key).unwrap();
-            let mut lengths: HashMap<usize, Vec<_>> = HashMap::new();
-            let mut elems = Vec::new();
-            for (vector_key, vector_data) in &vectors_to_add.vectors {
-                let vector = vector_data.vector.clone();
-                let bucket = lengths.entry(vector.len()).or_default();
-                let labels = LabelDictionary::new(vector_data.labels.clone());
-                elems.push(Elem::new(vector_key.clone(), vector, labels, None));
-                bucket.push(vector_key.clone());
-            }
-            if !elems.is_empty() {
-                let similarity = writer.metadata().similarity;
-                let location = writer.location();
-                let new_data_point_pin = DataPointPin::create_pin(location)?;
-                data_point::create(&new_data_point_pin, elems, Some(temporal_mark), similarity)?;
-                writer.add_data_point(new_data_point_pin)?;
-            }
-        }
-
-        for (_, mut writer) in writer_sets {
-            writer.commit()?;
-        }
-
-        let v = time.elapsed().as_millis();
-        debug!("{id:?} - Set resource in vector sets: ends {v} ms");
-
         let metrics = metrics::get_metrics();
         let took = time.elapsed().as_secs_f64();
         let metric = request_time::RequestTimeKey::vectors("set_resource".to_string());
@@ -290,31 +188,12 @@ impl VectorWriter for VectorWriterService {
     }
 
     fn get_segment_ids(&self) -> NodeResult<Vec<String>> {
-        let mut segment_ids = replication::get_segment_ids(&self.config.path)?;
-        for vs in self.list_vectorsets()? {
-            segment_ids.extend(replication::get_segment_ids(&self.config.vectorset.join(vs))?);
-        }
-        Ok(segment_ids)
+        Ok(replication::get_segment_ids(&self.config.path)?)
     }
 
     fn get_index_files(&self, ignored_segment_ids: &[String]) -> NodeResult<IndexFiles> {
         // Should be called along with a lock at a higher level to be safe
-        let mut replica_state = replication::get_index_files(&self.config.path, "vectors", ignored_segment_ids)?;
-
-        let vectorsets = self.list_vectorsets()?;
-        if !vectorsets.is_empty() {
-            replica_state
-                .metadata_files
-                .insert("vectorset/state.bincode".to_string(), fs::read(self.config.vectorset.join("state.bincode"))?);
-            for vs in vectorsets {
-                let vectorset_replica_state = replication::get_index_files(
-                    &self.config.vectorset.join(&vs),
-                    &format!("vectorset/{vs}"),
-                    ignored_segment_ids,
-                )?;
-                replica_state.extend(vectorset_replica_state);
-            }
-        }
+        let replica_state = replication::get_index_files(&self.config.path, "vectors", ignored_segment_ids)?;
 
         if replica_state.files.is_empty() {
             // exit with no changes
@@ -356,7 +235,6 @@ impl VectorWriterService {
     #[tracing::instrument(skip_all)]
     pub fn new(config: &VectorConfig) -> NodeResult<Self> {
         let path = std::path::Path::new(&config.path);
-        let indexset = std::path::Path::new(&config.vectorset);
         if path.exists() {
             Err(node_error!("Shard does exist".to_string()))
         } else {
@@ -366,10 +244,10 @@ impl VectorWriterService {
             let index_metadata = IndexMetadata {
                 similarity,
                 channel: config.channel,
+                normalize_vectors: config.normalize_vectors,
             };
             Ok(VectorWriterService {
                 index: Writer::new(path, index_metadata, config.shard_id.clone())?,
-                indexset: WriterSet::new(indexset, config.shard_id.clone())?,
                 config: config.clone(),
             })
         }
@@ -378,13 +256,11 @@ impl VectorWriterService {
     #[tracing::instrument(skip_all)]
     pub fn open(config: &VectorConfig) -> NodeResult<Self> {
         let path = std::path::Path::new(&config.path);
-        let indexset = std::path::Path::new(&config.vectorset);
         if !path.exists() {
             Err(node_error!("Shard does not exist".to_string()))
         } else {
             Ok(VectorWriterService {
                 index: Writer::open(path, config.shard_id.clone())?,
-                indexset: WriterSet::new(indexset, config.shard_id.clone())?,
                 config: config.clone(),
             })
         }
@@ -395,103 +271,13 @@ impl VectorWriterService {
 mod tests {
     use nucliadb_core::protos::resource::ResourceStatus;
     use nucliadb_core::protos::{
-        IndexParagraph, IndexParagraphs, Resource, ResourceId, UserVector, UserVectors, VectorSearchRequest,
-        VectorSentence, VectorSimilarity,
+        IndexParagraph, IndexParagraphs, Resource, ResourceId, VectorSentence, VectorSimilarity,
     };
     use nucliadb_core::Channel;
-    use std::collections::{HashMap, HashSet};
+    use std::collections::HashMap;
     use tempfile::TempDir;
 
     use super::*;
-    use crate::service::reader::VectorReaderService;
-    fn create_vector_set(writer: &mut VectorWriterService, set_name: String) -> (String, UserVectors) {
-        let label = format!("{set_name}/label");
-        let key = format!("{set_name}/key");
-        let vector = vec![1.0, 3.0, 4.0];
-        let data = UserVector {
-            vector,
-            labels: vec![label],
-            ..Default::default()
-        };
-        let set = UserVectors {
-            vectors: HashMap::from([(key, data)]),
-        };
-        let id = VectorSetId {
-            shard: None,
-            vectorset: set_name.clone(),
-        };
-        writer.add_vectorset(&id, VectorSimilarity::Cosine).unwrap();
-        (set_name, set)
-    }
-    #[test]
-    fn test_vectorset_functionality() {
-        let dir = TempDir::new().unwrap();
-        let vsc = VectorConfig {
-            similarity: Some(VectorSimilarity::Cosine),
-            path: dir.path().join("vectors"),
-            vectorset: dir.path().join("vectorsets"),
-            channel: Channel::EXPERIMENTAL,
-            shard_id: "abc".into(),
-        };
-
-        let mut writer = VectorWriterService::start(&vsc).expect("Error starting vector writer");
-        let indexes: HashMap<_, _> = (0..10).map(|i| create_vector_set(&mut writer, i.to_string())).collect();
-        let keys: HashSet<_> = indexes.keys().cloned().collect();
-        let resource = Resource {
-            vectors: indexes,
-            ..Default::default()
-        };
-        writer.set_resource(&resource).unwrap();
-        let index_keys: HashSet<_> = writer.list_vectorsets().unwrap().into_iter().collect();
-        assert_eq!(index_keys, keys);
-
-        let mut reader = VectorReaderService::start(&vsc).unwrap();
-        let mut request = VectorSearchRequest {
-            id: "".to_string(),
-            vector_set: "4".to_string(),
-            vector: vec![4.0, 6.0, 7.0],
-            field_labels: vec!["4/label".to_string()],
-            page_number: 0,
-            result_per_page: 20,
-            with_duplicates: false,
-            ..Default::default()
-        };
-        let context = VectorsContext::default();
-        let results = reader.search(&request, &context).unwrap();
-        let id = results.documents[0].doc_id.clone().unwrap().id;
-        assert_eq!(results.documents.len(), 1);
-        assert_eq!(id, "4/key");
-
-        // Same set, but no label match
-        request.field_labels = vec!["5/label".to_string()];
-        let results = reader.search(&request, &context).unwrap();
-        assert_eq!(results.documents.len(), 0);
-
-        // Invalid set
-        request.vector_set = "not a set".to_string();
-        let results = reader.search(&request, &context).unwrap();
-        assert_eq!(results.documents.len(), 0);
-
-        // Remove set 4
-        let id = VectorSetId {
-            vectorset: "4".to_string(),
-            ..Default::default()
-        };
-        let mut index_keys: HashSet<_> = writer.list_vectorsets().unwrap().into_iter().collect();
-        writer.remove_vectorset(&id).unwrap();
-        let index_keysp: HashSet<_> = writer.list_vectorsets().unwrap().into_iter().collect();
-        index_keys.remove("4");
-        assert!(!index_keysp.contains("4"));
-        assert_eq!(index_keys, index_keysp);
-
-        // Now vectorset 4 is no longer available
-        request.vector_set = "4".to_string();
-        request.field_labels = vec!["4/label".to_string()];
-
-        reader.update().unwrap();
-        let results = reader.search(&request, &context).unwrap();
-        assert_eq!(results.documents.len(), 0);
-    }
 
     #[test]
     fn test_new_vector_writer() {
@@ -499,9 +285,9 @@ mod tests {
         let vsc = VectorConfig {
             similarity: Some(VectorSimilarity::Cosine),
             path: dir.path().join("vectors"),
-            vectorset: dir.path().join("vectorset"),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
+            normalize_vectors: false,
         };
         let raw_sentences = [
             ("DOC/KEY/1/1".to_string(), vec![1.0, 3.0, 4.0]),
@@ -566,9 +352,9 @@ mod tests {
         let vsc = VectorConfig {
             similarity: Some(VectorSimilarity::Cosine),
             path: dir.path().join("vectors"),
-            vectorset: dir.path().join("vectorset"),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
+            normalize_vectors: false,
         };
         let resource_id = ResourceId {
             shard_id: "DOC".to_string(),
